      PROGRAM HEATCYCL
C
C  This program tryes to solve some of the deficiencies that INDHEAT had
C  for the calculation of the temperature evolution during induction 
C  heating and water spray quenching.
C 
C  It is based on the finite differences method. The implementation used 
C  is that developed by J. Crank and P. Nicholson. [al]
C
C	Program written March 2000 by D. Gaude-Fugarolas
C
C
C     ...Variable declaration...
C
      INTEGER IIU, IOU, ERR, INIOUT
      CHARACTER*55 FILE1, FILE2
C
      INTEGER I, J, K, N, IMAX, JMAX, CTRL, SYMM
      PARAMETER (ITERMAX = 100)
      PARAMETER (JMAX = 21)
      PARAMETER (IMAX = 100000)
      PARAMETER (KMAX = 10000)
      PARAMETER (IIU = 1)
      PARAMETER (IOU = 2)
C
      DOUBLE PRECISION TNOW(0:(JMAX-1)), UNOW(0:(JMAX-1)), 
     *     UNEXT(0:(JMAX-1)), DA(0:(JMAX-1)), LA(0:(JMAX-1)), 
     *     UA(0:(JMAX-1)), B(0:(JMAX-1))
      DOUBLE PRECISION TIME, INCTIME, INCTHT, L, QBIG, H, 
     *     P, TI, UTEMP, EXTRA, INCX, TOPT, DEADTIME, ELPSDTIME
      DOUBLE PRECISION RHO, KTH, HAIR, HWAT, CP, TMIN, TMAX
      DOUBLE PRECISION NORMT, NORMQ, GETQ, UNNORMT
C
      ERR = 0
      INIOUT = 0
      TIME = 0.0
C
C     
C    ...get input data from a file...
C
C
      N=JMAX
      CALL GETDAT (IIU, N, RHO, KTH, HAIR, HWAT, CP, TMIN,
     *     TMAX, TI, L, INCTIME, TOPT, DEADTIME, FILE1, SYMM, ERR)
C
      IF (ERR.EQ.1) THEN
         PRINT *,'N should be lower than', (JMAX)
         GOTO 99
      ENDIF
C
C  ...Opens output file and prepares to add results in there...
C
      CALL OUTDAT(IOU, N, TIME, TNOW, FILE2, INIOUT)

C  ...First iteration, normalising, etc CTRL=1 => Induction Heating...
C
      CTRL = 1
      INCTHT = (KTH*INCTIME)/(RHO*CP*L*L)
      INCX = 1.0/N
      P = INCTHT/(INCX*INCX)
      UTEMP = NORMT (TMAX, TMIN, TI)
C
C   ...Assign values to the norm and unnorm temperature vectors...
C
C
      DO 10, J = 0, (N)
         UNOW(J) = UTEMP
 10   CONTINUE
C
      DO 20, J = 0, (N)
         TNOW(J) = TI
 20   CONTINUE
C      
C   ...Print first row of temperatures to file (initial values)
C
      INIOUT = 1
      CALL OUTDAT(IOU, N, TIME, TNOW, FILE2, INIOUT)
C
C  ...Calculations for next step start...
C
C   ...if heat flux varies, the next line should be within the loop...
C
      QBIG = GETQ()
C
      PRINT *,'induction heating'
      DO 50 I = 1, IMAX
C
C
         EXTRA = NORMQ(QBIG, INCTHT, L, KTH, TMAX, TMIN, SYMM) 
C
         CALL EVALB(N, UNOW, P, EXTRA, CTRL, SYMM, B)
C
         CALL EVALA(N, P, EXTRA, CTRL, SYMM, DA, UA, LA) 
C
         CALL D3SOLV((N+1), DA, UA, LA, B, UNEXT)
C
         TIME = TIME + INCTIME
C
C
         DO 30 J = 0, N
C
            TNOW(J) = UNNORMT(TMAX, TMIN, UNEXT(J))
            UNOW(J) = UNEXT(J)
C
 30      CONTINUE
C
         CALL OUTDAT(IOU, N, TIME,  TNOW, FILE2, INIOUT)
C
C   ...Condition of end of INDUCTION HEATING...
C
         IF (UNOW(N).GE.(NORMT(TMAX, TMIN, TOPT))) GOTO 60
C
 50   CONTINUE
C
C   ...If the program arrives here, there's something wrong...
C   ...Like for instance the maximum of iterations has been reached...
C
      GOTO 98
C
C
 60   print *,'dead time cooling'
      CTRL = 2
      H = HAIR
      ELPSDTIME = 0.0
C
      DO 70, K=0, KMAX
C
         EXTRA = (H*L*INCX)/(2.0*KTH)
C
         CALL EVALB(N, UNOW, P, EXTRA, CTRL, SYMM, B)
C
         CALL EVALA(N, P, EXTRA, CTRL, SYMM, DA, UA, LA) 
C
         CALL D3SOLV((N+1), DA, UA, LA, B, UNEXT)
C
         TIME = TIME + INCTIME
         ELPSDTIME = ELPSDTIME + INCTIME
C
C   ...Condition of end of deadtime...
C
         IF((ELPSDTIME.GE.DEADTIME).AND.
     *      ((ELPSDTIME.LT.(DEADTIME+INCTIME)))) THEN
           H = HWAT
           print *,'water spray active'
         ENDIF
C
         DO 35 J = 0, N
C
            TNOW(J) = UNNORMT(TMAX, TMIN, UNEXT(J))
            UNOW(J) = UNEXT(J)
C
 35      CONTINUE
C
         CALL OUTDAT(IOU, N, TIME, TNOW, FILE2, INIOUT)
C
C   ...Condition of end of cooling...
C   ...defined as 1% of range of temperatures (Tmax-Tmin)...
C
         IF((TNOW(N)-TI).LT.(0.01*(TMAX-TMIN))) GOTO 96
C
 70      CONTINUE
C
      PRINT *,'Max number of iterations reached,IMAX=', KMAX
C*
 96   INIOUT = 2
      CALL OUTDAT(IOU, N, TIME,  TNOW, FILE2, INIOUT)
      PRINT *,'Calculations finished. Output file is:', FILE2
C
      STOP
C
 98   PRINT *,'Max number of iterations reached, IMAX=', IMAX
C
 99   PRINT *,'Unable to continue calculations'
      STOP
      END
C    *******************************************************************
C    *******************************************************************
C
C    Subroutine GETDAT to read from a file the parameters to run the
C    model
C
      SUBROUTINE GETDAT  (IIU, N, RHO, KTH, HAIR, HWAT, CP, TMIN,
     *     TMAX, TI, L, INCTIME, TOPT, DEADTIME, FILE1, SYMM, ERR)
C
C
      INTEGER IIU, N, ERR, SYMM, JMAX
      DOUBLE PRECISION RHO, KTH, HAIR, HWAT, CP, TMIN, TMAX, TI,
     *     INCTIME, L, TOPT, DEADTIME
      CHARACTER*55 FILE1
C
C
C
      JMAX = N
C
      WRITE(*,*) 'Input file name?'
      READ (*,*) FILE1
      OPEN (UNIT = IIU, FILE = FILE1, STATUS = 'OLD')
C
      READ (IIU, *) N
      IF (N.GT.(JMAX-1)) THEN
         ERR = 1
      ENDIF
      READ (IIU, *) TMIN, TMAX, TI
      READ (IIU, *) KTH, HAIR, HWAT, RHO, CP
      READ (IIU, *) L, SYMM
      READ (IIU, *) INCTIME
      READ (IIU, *) TOPT
      READ (IIU, *) DEADTIME
C
      CLOSE (IIU)
C
      RETURN
      END
C
C     *****************************************************************
C
C     Double precision function NORMT(TMAX, TMIN, T)
C
C     normalises temperatures
C
      DOUBLE PRECISION FUNCTION NORMT(TMAX, TMIN, T)
C
C
      DOUBLE PRECISION TMAX, TMIN, T
C
C
      NORMT = (T-TMIN)/(TMAX-TMIN)
C
      RETURN
      END
C
C     *****************************************************************
C
      DOUBLE PRECISION FUNCTION NORMQ(Q, INCTHT, L, KTH, TX, TN, S)
C
C
C     Nomalises the heat addition for plate or cylinder
C      S = 1 plate; S = 0 cylinder
C
      INTEGER S
      DOUBLE PRECISION Q, INCTHT, KTH, TX, TN, L
C
C
      IF(S.EQ.1) THEN
         NORMQ = (INCTHT * Q *L)/(KTH*(TX-TN))
      ELSE IF(S.EQ.0) THEN
         NORMQ = (INCTHT * Q)/(KTH*(TX-TN))
      ENDIF    
C
C
      RETURN
      END
C
C     *****************************************************************
C
C
C
      DOUBLE PRECISION FUNCTION GETQ ()
C
C
      DOUBLE PRECISION Q
      WRITE(*,*)'Input heat flux'
      READ (*,*) Q
      GETQ = Q
      RETURN
      END
C
C     ****************************************************************
C
C
      SUBROUTINE EVALB(N, U, P, EXTRA, C, S, B)
C
C     Evaluation of independent term of simultaneous eq. system
C
C     for either plate (s=1), or cylinder (s=0), and during
C     induction heating (CTRL = 1) or cooling (CTRL = 2)
C
      INTEGER N, C, S, I
      DOUBLE PRECISION U(0:(N)), B(0:(N)), P, EXTRA
C
      DOUBLE PRECISION UT, LT, DT, COOL
C
C
      IF (C.EQ.1) THEN
         U(N) = U(N) + EXTRA
         COOL = 1.0
      ELSE IF (C.EQ.2) THEN
         COOL = 1.0 + EXTRA
      ENDIF
C
C
C PLATE AND CYLINDER
C
      I = 0
      B(I) = (1.0 - P)*U(I) + P*U(I+1) 
C
      DO 12, I=1, (N-1)
         DT = (1.0 - P)
         IF (S.EQ.1) THEN 
            UT = P/2
            LT = P/2
         ELSE IF(S.EQ.0) THEN
            UT = ((2*I+1.0)/(4*I))*P
            LT = ((2*I-1.0)/(4*I))*P
         ENDIF
         B(I) = LT*U(I-1) + DT*U(I) + UT*U(I+1)
 12   CONTINUE
      IF(S.EQ.1) THEN
         B(N) = P*U(N-1) + (1.0 - P*COOL)*U(N)
      ELSE IF(S.EQ.0) THEN
         B(N)=((2.*N+1.)*P/(2.*N))*U(N-1)+
     *      (1.0-((2.*N+1.)*P/(2.*N))*COOL)*U(N)
      ENDIF
C
C
      RETURN
      END
C
C
C     *****************************************************************
C
C
      SUBROUTINE EVALA(N, P, EXTRA, C, S, DA, UA, LA)
C
C     Calculates the three diagonals of the (v+1) matrix, ans stores
C     them in the vectors provided (UA, DA, LA)
C
C
      INTEGER N, C, S, I
      DOUBLE PRECISION P, EXTRA, COOL
      DOUBLE PRECISION DA(0:(N)), UA(0:(N)), LA(0:(N))
C
C
      IF (C.EQ.1) THEN
         COOL = 1.0
      ELSE IF (C.EQ.2) THEN
         COOL = 1.0 + EXTRA
      ENDIF
C
C
      I=0
      DA(I) = 1+P
      LA(I) = 0.0
      UA(I) = -P
      DO 23, I = 1, N-1
         DA(I) = 1+P
         IF(S.EQ.1) THEN
            LA(I) = -P/2.0
            UA(I) = -P/2.0
         ELSE IF (S.EQ.0) THEN
            LA(I) = -((2.0*I - 1.0)/(4.0*I))*P
            UA(I) = -((2.0*I + 1.0)/(4.0*I))*P
         ENDIF
 23   CONTINUE
C
      IF(S.EQ.1) THEN
         LA(N) = -P
         UA(N) = 0.0
         DA(N) = 1.0 + P*COOL
      ELSE IF(S.EQ.0) THEN
         LA(N) = (2.0*N+1.0)*P/(2.0*N)
         UA(N) = 0
         DA(N) = 1.0-((2.0*N+1.0)*P/(2.0*N))*COOL
      ENDIF

C
C
      RETURN
      END
C
C     *************************************************************
C     *************************************************************
C
C
      SUBROUTINE D3SOLV(N, DA, UA, LA, B, X)
C
C
C      Writen by Daniel Gaude-Fugarolas in February 2000
C
C     Solves tridiagonal simultaneous linear systems AX=B
C
C     Main diagonal...DA(1-n)  Upper/Lower...UA(1-(n-1))/LA(2-n)
C
C     In two steps, first prepares matrices, and then solves.
C
C     ...dummie arguments...
C
      INTEGER N
      DOUBLE PRECISION DA(N), UA(N), LA(N), B(N), X(N)
C
C     ...Local arguments...
C
      INTEGER I
      DOUBLE PRECISION TEMP
C
C     ...Routine starts...
C     ...First modifies matrices...
C
      TEMP = DA(1)
      B(1) = B(1) / TEMP
      UA(1) = UA(1) / TEMP
C
      DO 11, I=2, (N-1)
           TEMP = DA(I) - LA(I)*UA(I-1)
           B(I) = B(I) - B(I-1)*LA(I)
           UA(I) = UA(I) / TEMP
           B(I) = B(I) / TEMP
C
  11  CONTINUE
C
      TEMP = DA(N) - LA(N)*UA(N-1)
      B(N) = B(N) - B(N-1)*LA(N)
      B(N) = B(N) / TEMP
C
C
C     ...Now solves the system...
C          ...from N to 1...
C
      X(N) = B(N)
      DO 21, I=(N-1), 1, -1
           X(I) = B(I) - UA(I)*X(I+1)
  21  CONTINUE
C
C     ...End of the routine...
C
      RETURN
      END
C
C
C     *******************************************************************
C     *******************************************************************
C
C
      SUBROUTINE OUTDAT(IOU, N, TIME, T, FILE2, INIOUT)
C
C
C     Output of results. 
C     With iniout = 0, asks for file name and opens it
C     With iniout = 1, adds a new line to the file (time and temperatures)
C     with iniout = 2, closes file
C
C
C
      INTEGER IOU, N, INIOUT, I
      DOUBLE PRECISION TIME, T(0:N)
      CHARACTER*55 FILE2
C
C   ...If iniout = 0, only asks for file name and opens it...
C
      IF(INIOUT.EQ.0) THEN 
         WRITE(*,*) 'Output file name?'
         READ (*,*) FILE2
         OPEN (UNIT = IOU, FILE = FILE2, STATUS = 'NEW')
         WRITE(IOU, 1001)
C
C   ...if iniout = 1 adds data to file...
C
      ELSE IF(INIOUT.EQ.1) THEN
         WRITE(IOU, 1002) TIME, (T(I), I=0, N)
C
C   ...if iniout = 2, closes file...
C
      ELSE IF(INIOUT.EQ.2) THEN
         CLOSE (IOU)
      END IF
C
      RETURN
C   ...Formats...
C
 1001 FORMAT(1H , 'Time /s', 5X, '... Temperatures /C ...',5X, 
     *     'from center to surface')
 1002 FORMAT(1X,F8.4,3X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,
     * F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,
     * F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1,2X,F6.1)
C
      END
C
C
C ***********************************************************************
C     *******************************************************************
C
      DOUBLE PRECISION FUNCTION UNNORMT(TX, TN, U)
C
C
      DOUBLE PRECISION TX, TN, U
C
      UNNORMT = U*(TX-TN) + TN
C
      RETURN
      END
C
C
C     *******************************************************************
C




